// ----------------------------------------------------------------------------
//
// Kurve91.PAS                                            Ulrich Mller  7.3.99
//                                                        Procida 400NT Delphi4
// fischertechnik Computing Kurvenschreiber 91 : Adaption des DOS TurboPascal
// Programmes an Windows und Delphi 4
// erforderlich : umFish.DLL
// bei Einsatz des parallelen Interfaces ggf. LPTDelay, LPTAnalog anpassen
// ebenso : Const Min/Max_Analogwert
//
// ----------------------------------------------------------------------------

unit Kurve91;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, jpeg;

type
  TfrmKurve91 = class(TForm)
    cmdAction: TButton;
    cmdHalt: TButton;
    cmdEnde: TButton;
    lblTitel: TLabel;
    lblCopyright: TLabel;
    Label1: TLabel;
    lblMesswert: TLabel;
    Image1: TImage;
    Label2: TLabel;
    Label3: TLabel;
    lblZielPos: TLabel;
    lblAktPos: TLabel;
    cboPort: TComboBox;
    procedure cmdEndeClick(Sender: TObject);
    procedure cmdHaltClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmdActionClick(Sender: TObject);
  private
    { Private-Deklarationen}
  public
    { Public-Deklarationen}
  end;

  type ftDCB = record
    hCom: LongInt;
    PortID: LongInt;
    LPTDelay: LongInt;
    LPTAnalog: LongInt;
    InputStatus: LongInt;
    MotorStatus: LongInt;
  end;
  type pftDCB = ^ftDCB;

  function OpenInterface(ft: pftDCB; PortName: string):LongInt;
       stdcall; external 'umFish.dll';
  function CloseInterface(ft: pftDCB):LongInt;
       stdcall; external 'umFish.dll';
  function GetInput(ft: pftDCB; InputNr:LongInt):LongInt;
       stdcall; external 'umFish.dll';
  function GetInputs(ft: pftDCB):LongInt;
       stdcall; external 'umFish.dll';
  function ClearMotors(ft: pftDCB):LongInt;
       stdcall; external 'umFish.dll';
  function SetMotors(ft: pftDCB):LongInt;
       stdcall; external 'umFish.dll';
  function SetMotor(ft: pftDCB; MotorNr, Direction:LongInt):LongInt;
       stdcall; external 'umFish.dll';
  function GetAnalog(ft: pftDCB; AnalogNr:LongInt):LongInt;
       stdcall; external 'umFish.dll';
const
     ftiAus=0; ftiLinks=1; ftiRechts=2;
     ftiFehler=$0001FFFF; ftiTrue=-1; ftiFalse=0;
var
  frmKurve91: TfrmKurve91;
  FishFace: ftDCB; pFishFace: pftDCB;

implementation

{$R *.DFM}

const
   cSchreiMotor = 1; cSchreiImpuls = 1; cSchreiEnde = 4;
   cStiftMotor  = 2; cStiftOben    = 2;
   cPapierMotor = 3; cPapierImpuls = 3;
   cSensor      = 0;
   Minimaler_Analogwert_Sensor = 0;
   Maximaler_Analogwert_Sensor = 1024;
   Analoger_Messbereich = Maximaler_Analogwert_Sensor - Minimaler_Analogwert_Sensor;
   Maximale_Schrei_Schritte = 360;
   ASfaktor = Analoger_Messbereich div Maximale_Schrei_Schritte;
var
  Stop: Boolean;               // Abbruch Programmschleife
  X: LongInt;                  // letzte Position
  Zielposition_X: LongInt;     // neue X-Position
  Analogwert: LongInt;         // Mewert
  SekundenMerker: Word;        // fr Vorsuch alle Sekunden
  Std, Min, Sek, MSek: Word;

procedure Analogeingang_einlesen_und_verarbeiten;
  begin
    Analogwert := GetAnalog(pFishFace,cSensor);  // Einlesen Sensorwert
    Zielposition_X := (Analogwert - Minimaler_Analogwert_Sensor) div ASfaktor;
    if Zielposition_X > X then                  // Ein Schritt nach X+
      begin
      SetMotor(pFishFace, cSchreiMotor, ftiLinks);
      repeat until GetInput(pFishFace, cSchreiImpuls) = ftiTrue;
      repeat until GetInput(pFishFace, cSchreiImpuls)= ftiFalse;
      SetMotor(pFishFace, cSchreiMotor, ftiAus);
      inc(X);
      end;
    if Zielposition_X < X then                 // Ein Schritt nach X-
      begin
      SetMotor(pfishFace, cSchreiMotor, ftiRechts);
      repeat until GetInput(pFishFace, cSchreiImpuls) = ftiTrue;
      repeat until GetInput(pFishFace, cSchreiImpuls)= ftiFalse;
      SetMotor(pFishFace, cSchreiMotor, ftiAus);
      dec(X);
      end;

  // --- Anzeige der Werte ---

    frmKurve91.lblMesswert.Caption := IntToStr(Analogwert);
    frmKurve91.lblZielPos.Caption  := IntToStr(Zielposition_X);
    frmKurve91.lblAktPos.Caption   := IntToStr(X);

  end;

procedure Kennlinienschreiber_initialisieren;
  begin
    Stop := false;
    SetMotor(pFishFace, cSchreiMotor, ftiRechts);  // Schreiber zum Endtaster
    repeat Application.ProcessMessages; until (GetInput(pFishFace, cSchreiEnde) = ftiTrue) or Stop;
    SetMotor(pFishFace, cSchreiMotor, ftiAus);
    SetMotor(pFishFace, cStiftMotor, ftiRechts);  // Stift senken
    repeat Application.ProcessMessages; until (GetInput(pFishFace, cStiftOben) = ftiFalse) or Stop = true;
    SetMotor(pFishFace, cStiftMotor, ftiAus);

//  --- Ausgangswerte ---

    X := 0;
    SekundenMerker := 0;
    frmKurve91.lblMesswert.Caption := '???';
    frmKurve91.lblZielPos.Caption  := '???';
    frmKurve91.lblAktPos.Caption   := '???';
  end;

procedure Zeilenvorschub;
  begin
    DecodeTime(Time, Std, Min, Sek, MSek);
    if Sek <> SekundenMerker then    // Papiervorschub, ein Schritt
    begin
    SetMotor(pFishFace, cPapierMotor, ftiRechts);
    repeat until GetInput(pFishFace, cPapierImpuls) = ftiTrue;
    repeat until GetInput(pFishFace, cPapierImpuls) = ftiFalse;
    SetMotor(pFishFace, cPapierMotor, ftiAus);
    SekundenMerker := Sek;           // neue Vorschubzeit
    end;

  end;

procedure TfrmKurve91.cmdEndeClick(Sender: TObject);
begin
     close;
end;

procedure TfrmKurve91.cmdHaltClick(Sender: TObject);
begin
   Stop := true;                    // Loop-Ende : Drcken HALT-Button
end;

procedure TfrmKurve91.FormCreate(Sender: TObject);
begin
   pFishFace := @FishFace;
   cboPort.ItemIndex := 0;          // --- COM1 = Default
   Stop := true;
end;

procedure TfrmKurve91.cmdActionClick(Sender: TObject);

begin
   cmdHalt.SetFocus;
   OpenInterface(pFishFace, cboPort.Text);
   Kennlinienschreiber_initialisieren;
   if not Stop then
   begin
   repeat
     Analogeingang_einlesen_und_verarbeiten;
     Zeilenvorschub;
     Application.ProcessMessages;
   until Stop;                      // --- Loop-Ende : HALT-Button
   SetMotor(pFishFace, cStiftMotor, ftiLinks);  // Stift heben
   repeat until GetInput(pFishFace, cStiftOben) = ftiTrue;
   SetMotor(pFishFace, cStiftMotor, ftiAus);
   end;
   cmdAction.SetFocus;
   CloseInterface(pFishFace);
end;

end.
